@@ -22,7 +22,7 @@ from PyQt5.Qt import (
 
 from calibre import prints, force_unicode, detect_ncpus
 from calibre.constants import (
-        __appname__, isosx, iswindows, filesystem_encoding, DEBUG)
+        __appname__, isosx, iswindows, filesystem_encoding, DEBUG, config_dir)
 from calibre.utils.config import prefs, dynamic
 from calibre.utils.ipc.pool import Pool
 from calibre.db.legacy import LibraryDatabase
@@ -31,6 +31,7 @@ from calibre.gui2 import (error_dialog, GetMetadata, open_url,
         gprefs, max_available_height, config, info_dialog, Dispatcher,
         question_dialog, warning_dialog)
 from calibre.gui2.cover_flow import CoverFlowMixin
+from calibre.gui2.changes import handle_changes
 from calibre.gui2.widgets import ProgressIndicator
 from calibre.gui2.update import UpdateMixin
 from calibre.gui2.main_window import MainWindow
@@ -51,6 +52,7 @@ from calibre.gui2.job_indicator import Pointer
 from calibre.gui2.dbus_export.widgets import factory
 from calibre.gui2.open_with import register_keyboard_shortcuts
 from calibre.library import current_library_name
+from calibre.srv.library_broker import GuiLibraryBroker
 
 
 class Listener(Thread):  # {{{
@@ -85,11 +87,8 @@ class Listener(Thread):  # {{{
 # }}}
 
 
-_gui = None
-
-
 def get_gui():
-    return _gui
+    return getattr(get_gui, 'ans', None)
 
 
 def add_quick_start_guide(library_view, refresh_cover_browser=None):
@@ -136,7 +135,6 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
     shutting_down = False
 
     def __init__(self, opts, parent=None, gui_debug=None):
-        global _gui
         MainWindow.__init__(self, opts, parent=parent, disable_automatic_gc=True)
         self.setWindowIcon(QApplication.instance().windowIcon())
         self.jobs_pointer = Pointer(self)
@@ -145,7 +143,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         self.proceed_question = ProceedQuestion(self)
         self.job_error_dialog = JobError(self)
         self.keyboard = Manager(self)
-        _gui = self
+        get_gui.ans = self
         self.opts = opts
         self.device_connected = None
         self.gui_debug = gui_debug
@@ -220,7 +218,11 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         opts = self.opts
         self.preferences_action, self.quit_action = actions
         self.library_path = library_path
+        self.library_broker = GuiLibraryBroker(db)
         self.content_server = None
+        self.server_change_notification_timer = t = QTimer(self)
+        self.server_changes = Queue()
+        t.setInterval(1000), t.timeout.connect(self.handle_changes_from_server_debounced), t.setSingleShot(True)
         self._spare_pool = None
         self.must_restart_before_config = False
         self.listener = Listener(listener)
@@ -246,7 +248,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         # Jobs Button {{{
         self.job_manager = JobManager()
         self.jobs_dialog = JobsDialog(self, self.job_manager)
-        self.jobs_button = JobsButton(horizontal=True, parent=self)
+        self.jobs_button = JobsButton(parent=self)
         self.jobs_button.initialize(self.jobs_dialog, self.job_manager)
         # }}}
 
@@ -263,25 +265,23 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         self.tb_wrapper = textwrap.TextWrapper(width=40)
         self.viewers = collections.deque()
         self.system_tray_icon = None
-        if config['systray_icon']:
+        do_systray = config['systray_icon'] or opts.start_in_tray
+        if do_systray:
             self.system_tray_icon = factory(app_id='com.calibre-ebook.gui').create_system_tray_icon(parent=self, title='calibre')
         if self.system_tray_icon is not None:
             self.system_tray_icon.setIcon(QIcon(I('lt.png', allow_user_override=False)))
             if not (iswindows or isosx):
-                self.system_tray_icon.setIcon(QIcon.fromTheme('calibre-gui', self.system_tray_icon.icon()))
+                self.system_tray_icon.setIcon(QIcon.fromTheme('calibre-tray', self.system_tray_icon.icon()))
             self.system_tray_icon.setToolTip(self.jobs_button.tray_tooltip())
             self.system_tray_icon.setVisible(True)
             self.jobs_button.tray_tooltip_updated.connect(self.system_tray_icon.setToolTip)
-        elif config['systray_icon']:
-            prints('Failed to create system tray icon, your desktop environment probably does not support the StatusNotifier spec')
+        elif do_systray:
+            prints('Failed to create system tray icon, your desktop environment probably'
+                   ' does not support the StatusNotifier spec https://www.freedesktop.org/wiki/Specifications/StatusNotifierItem/')
         self.system_tray_menu = QMenu(self)
         self.toggle_to_tray_action = self.system_tray_menu.addAction(QIcon(I('page.png')), '')
         self.toggle_to_tray_action.triggered.connect(self.system_tray_icon_activated)
         self.system_tray_menu.addAction(self.donate_action)
-        self.donate_button.clicked.connect(self.donate_action.trigger)
-        self.donate_button.setToolTip(self.donate_action.text().replace('&', ''))
-        self.donate_button.setIcon(self.donate_action.icon())
-        self.donate_button.setStatusTip(self.donate_button.toolTip())
         self.eject_action = self.system_tray_menu.addAction(
                 QIcon(I('eject.png')), _('&Eject connected device'))
         self.eject_action.setEnabled(False)
@@ -401,8 +401,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         self.read_settings()
 
         self.finalize_layout()
-        if self.bars_manager.showing_donate:
-            self.donate_button.start_animation()
+        self.bars_manager.start_animation()
         self.set_window_title()
 
         for ac in self.iactions.values():
@@ -425,6 +424,14 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         if self.system_tray_icon is not None and self.system_tray_icon.isVisible() and opts.start_in_tray:
             self.hide_windows()
         self.auto_adder = AutoAdder(gprefs['auto_add_path'], self)
+
+        # Now that the gui is initialized we can restore the quickview state
+        # The same thing will be true for any action-based operation with a
+        # layout button
+        from calibre.gui2.actions.show_quickview import get_quickview_action_plugin
+        qv = get_quickview_action_plugin()
+        if qv:
+            qv.qv_button.restore_state()
         self.save_layout_state()
 
         # Collect cycles now
@@ -446,7 +453,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
                     'log will be displayed automatically.')%self.gui_debug, show=True)
 
     def esc(self, *args):
-        self.clear_button.click()
+        self.search.clear()
 
     def shift_esc(self):
         self.current_view().setFocus(Qt.OtherFocusReason)
@@ -473,19 +480,52 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         self.iactions['Connect Share'].set_smartdevice_action_state()
 
     def start_content_server(self, check_started=True):
-        from calibre.library.server.main import start_threaded_server
-        from calibre.library.server import server_config
-        self.content_server = start_threaded_server(
-                self.library_view.model().db, server_config().parse())
+        from calibre.srv.embedded import Server
+        if not gprefs.get('server3_warning_done', False):
+            gprefs.set('server3_warning_done', True)
+            if os.path.exists(os.path.join(config_dir, 'server.py')):
+                try:
+                    os.remove(os.path.join(config_dir, 'server.py'))
+                except EnvironmentError:
+                    pass
+                warning_dialog(self, _('Content server changed!'), _(
+                    'calibre 3 comes with a completely re-written content server.'
+                    ' As such any custom configuration you have for the content'
+                    ' server no longer applies. You should check and refresh your'
+                    ' settings in Preferences->Sharing->Sharing over the net'), show=True)
+        self.content_server = Server(self.library_broker, Dispatcher(self.handle_changes_from_server))
         self.content_server.state_callback = Dispatcher(
                 self.iactions['Connect Share'].content_server_state_changed)
         if check_started:
             self.content_server.start_failure_callback = \
                 Dispatcher(self.content_server_start_failed)
+        self.content_server.start()
+
+    def handle_changes_from_server(self, library_path, change_event):
+        if DEBUG:
+            prints('Received server change event: {} for {}'.format(change_event, library_path))
+        if self.library_broker.is_gui_library(library_path):
+            self.server_changes.put((library_path, change_event))
+            self.server_change_notification_timer.start()
+
+    def handle_changes_from_server_debounced(self):
+        if self.shutting_down:
+            return
+        changes = []
+        while True:
+            try:
+                library_path, change_event = self.server_changes.get_nowait()
+            except Empty:
+                break
+            if self.library_broker.is_gui_library(library_path):
+                changes.append(change_event)
+        if changes:
+            handle_changes(changes, self)
 
     def content_server_start_failed(self, msg):
-        error_dialog(self, _('Failed to start Content Server'),
-                _('Could not start the content server. Error:\n\n%s')%msg,
+        self.content_server = None
+        error_dialog(self, _('Failed to start Content server'),
+                _('Could not start the Content server. Error:\n\n%s')%msg,
                 show=True)
 
     def resizeEvent(self, ev):
@@ -558,13 +598,21 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
     def test_server(self, *args):
         if self.content_server is not None and \
                 self.content_server.exception is not None:
-            error_dialog(self, _('Failed to start content server'),
+            error_dialog(self, _('Failed to start Content server'),
                          unicode(self.content_server.exception)).exec_()
 
     @property
     def current_db(self):
         return self.library_view.model().db
 
+    def refresh_all(self):
+        m = self.library_view.model()
+        m.db.data.refresh(clear_caches=False, do_search=False)
+        self.saved_searches_changed(recount=False)
+        m.resort()
+        m.research()
+        self.tags_view.recount()
+
     def another_instance_wants_to_talk(self):
         try:
             msg = self.listener.queue.get_nowait()
@@ -593,10 +641,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         elif msg.startswith('refreshdb:'):
             m = self.library_view.model()
             m.db.new_api.reload_from_db()
-            m.db.data.refresh(clear_caches=False, do_search=False)
-            m.resort()
-            m.research()
-            self.tags_view.recount()
+            self.refresh_all()
         elif msg.startswith('shutdown:'):
             self.quit(confirm_quit=False)
         elif msg.startswith('bookedited:'):
@@ -631,81 +676,74 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
     def booklists(self):
         return self.memory_view.model().db, self.card_a_view.model().db, self.card_b_view.model().db
 
-    def library_moved(self, newloc, copy_structure=False, call_close=True,
-            allow_rebuild=False):
+    def library_moved(self, newloc, copy_structure=False, allow_rebuild=False):
         if newloc is None:
             return
-        default_prefs = None
-        try:
-            olddb = self.library_view.model().db
-            if copy_structure:
-                default_prefs = olddb.prefs
-        except:
-            olddb = None
-        if copy_structure and olddb is not None and default_prefs is not None:
-            default_prefs['field_metadata'] = olddb.new_api.field_metadata.all_metadata()
-        try:
-            db = LibraryDatabase(newloc, default_prefs=default_prefs)
-        except apsw.Error:
-            if not allow_rebuild:
-                raise
-            import traceback
-            repair = question_dialog(self, _('Corrupted database'),
-                    _('The library database at %s appears to be corrupted. Do '
-                    'you want calibre to try and rebuild it automatically? '
-                    'The rebuild may not be completely successful.')
-                    % force_unicode(newloc, filesystem_encoding),
-                    det_msg=traceback.format_exc()
-                    )
-            if repair:
-                from calibre.gui2.dialogs.restore_library import repair_library_at
-                if repair_library_at(newloc, parent=self):
-                    db = LibraryDatabase(newloc, default_prefs=default_prefs)
-                else:
-                    return
-            else:
-                return
-        if self.content_server is not None:
-            self.content_server.set_database(db)
-        self.library_path = newloc
-        prefs['library_path'] = self.library_path
-        self.book_on_device(None, reset=True)
-        db.set_book_on_device_func(self.book_on_device)
-        self.library_view.set_database(db)
-        self.tags_view.set_database(db, self.alter_tb)
-        self.library_view.model().set_book_on_device_func(self.book_on_device)
-        self.status_bar.clear_message()
-        self.search.clear()
-        self.saved_search.clear()
-        self.book_details.reset_info()
-        # self.library_view.model().count_changed()
-        db = self.library_view.model().db
-        self.iactions['Choose Library'].count_changed(db.count())
-        self.set_window_title()
-        self.apply_named_search_restriction('')  # reset restriction to null
-        self.saved_searches_changed(recount=False)  # reload the search restrictions combo box
-        if db.prefs['virtual_lib_on_startup']:
-            self.apply_virtual_library(db.prefs['virtual_lib_on_startup'])
-        self.rebuild_vl_tabs()
-        for action in self.iactions.values():
-            action.library_changed(db)
-        if olddb is not None:
+        with self.library_broker:
+            default_prefs = None
             try:
-                if call_close:
-                    olddb.close()
+                olddb = self.library_view.model().db
+                if copy_structure:
+                    default_prefs = olddb.prefs
             except:
-                import traceback
-                traceback.print_exc()
-            olddb.break_cycles()
-        if self.device_connected:
-            self.set_books_in_library(self.booklists(), reset=True)
-            self.refresh_ondevice()
-            self.memory_view.reset()
-            self.card_a_view.reset()
-            self.card_b_view.reset()
-        self.set_current_library_information(current_library_name(), db.library_id,
-                                             db.field_metadata)
-        self.library_view.set_current_row(0)
+                olddb = None
+            if copy_structure and olddb is not None and default_prefs is not None:
+                default_prefs['field_metadata'] = olddb.new_api.field_metadata.all_metadata()
+            db = self.library_broker.prepare_for_gui_library_change(newloc)
+            if db is None:
+                try:
+                    db = LibraryDatabase(newloc, default_prefs=default_prefs)
+                except apsw.Error:
+                    if not allow_rebuild:
+                        raise
+                    import traceback
+                    repair = question_dialog(self, _('Corrupted database'),
+                            _('The library database at %s appears to be corrupted. Do '
+                            'you want calibre to try and rebuild it automatically? '
+                            'The rebuild may not be completely successful.')
+                            % force_unicode(newloc, filesystem_encoding),
+                            det_msg=traceback.format_exc()
+                            )
+                    if repair:
+                        from calibre.gui2.dialogs.restore_library import repair_library_at
+                        if repair_library_at(newloc, parent=self):
+                            db = LibraryDatabase(newloc, default_prefs=default_prefs)
+                        else:
+                            return
+                    else:
+                        return
+            self.library_path = newloc
+            prefs['library_path'] = self.library_path
+            self.book_on_device(None, reset=True)
+            db.set_book_on_device_func(self.book_on_device)
+            self.library_view.set_database(db)
+            self.tags_view.set_database(db, self.alter_tb)
+            self.library_view.model().set_book_on_device_func(self.book_on_device)
+            self.status_bar.clear_message()
+            self.search.clear()
+            self.saved_search.clear()
+            self.book_details.reset_info()
+            # self.library_view.model().count_changed()
+            db = self.library_view.model().db
+            self.iactions['Choose Library'].count_changed(db.count())
+            self.set_window_title()
+            self.apply_named_search_restriction('')  # reset restriction to null
+            self.saved_searches_changed(recount=False)  # reload the search restrictions combo box
+            if db.prefs['virtual_lib_on_startup']:
+                self.apply_virtual_library(db.prefs['virtual_lib_on_startup'])
+            self.rebuild_vl_tabs()
+            for action in self.iactions.values():
+                action.library_changed(db)
+            self.library_broker.gui_library_changed(db, olddb)
+            if self.device_connected:
+                self.set_books_in_library(self.booklists(), reset=True)
+                self.refresh_ondevice()
+                self.memory_view.reset()
+                self.card_a_view.reset()
+                self.card_b_view.reset()
+            self.set_current_library_information(current_library_name(), db.library_id,
+                                                db.field_metadata)
+            self.library_view.set_current_row(0)
         # Run a garbage collection now so that it does not freeze the
         # interface later
         gc.collect()
@@ -750,7 +788,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         self.set_number_of_books_shown()
         self.update_status_bar()
 
-    def job_exception(self, job, dialog_title=_('Conversion Error'), retry_func=None):
+    def job_exception(self, job, dialog_title=_('Conversion error'), retry_func=None):
         if not hasattr(self, '_modeless_dialogs'):
             self._modeless_dialogs = []
         minz = self.is_minimized_to_tray
@@ -773,17 +811,17 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
                 title = job.description.split(':')[-1].partition('(')[-1][:-1]
                 msg = _('<p><b>Failed to convert: %s')%title
                 msg += '<p>'+_('''
-                Many older ebook reader devices are incapable of displaying
+                Many older e-book reader devices are incapable of displaying
                 EPUB files that have internal components over a certain size.
                 Therefore, when converting to EPUB, calibre automatically tries
                 to split up the EPUB into smaller sized pieces.  For some
                 files that are large undifferentiated blocks of text, this
                 splitting fails.
                 <p>You can <b>work around the problem</b> by either increasing the
-                maximum split size under EPUB Output in the conversion dialog,
+                maximum split size under <i>EPUB output</i> in the conversion dialog,
                 or by turning on Heuristic Processing, also in the conversion
                 dialog. Note that if you make the maximum split size too large,
-                your ebook reader may have trouble with the EPUB.
+                your e-book reader may have trouble with the EPUB.
                         ''')
                 if not minz:
                     d = error_dialog(self, _('Conversion Failed'), msg,
@@ -800,7 +838,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
                     idx = job.details.index('calibre.ebooks.mobi.reader.mobi6.KFXError:')
                     msg += '<p>' + re.sub(r'(https:\S+)', r'<a href="\1">{}</a>'.format(_('here')),
                                           job.details[idx:].partition(':')[2].strip())
-                    d = error_dialog(self, _('Conversion Failed'), msg, det_msg=job.details)
+                    d = error_dialog(self, _('Conversion failed'), msg, det_msg=job.details)
                     d.setModal(False)
                     d.show()
                     self._modeless_dialogs.append(d)
@@ -858,6 +896,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
             config.set('main_window_geometry', self.saveGeometry())
             dynamic.set('sort_history', self.library_view.model().sort_history)
             self.save_layout_state()
+            self.stack.tb_widget.save_state()
 
     def quit(self, checked=True, restart=False, debug_on_restart=False,
             confirm_quit=True):
@@ -890,7 +929,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
 
         if self.proceed_question.questions:
             msg = _('There are library updates waiting. Are you sure you want to quit?')
-            if not question_dialog(self, _('Library Updates Waiting'), msg):
+            if not question_dialog(self, _('Library updates waiting'), msg):
                 return False
 
         from calibre.db.delete_service import has_jobs
@@ -906,6 +945,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
     def shutdown(self, write_settings=True):
         self.shutting_down = True
         self.show_shutdown_message()
+        self.server_change_notification_timer.stop()
 
         from calibre.customize.ui import has_library_closed_plugins
         if has_library_closed_plugins():
@@ -955,17 +995,16 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         if mb is not None:
             mb.stop()
 
-        if db is not None:
-            db.close()
+        self.library_view.model().close()
 
         try:
             try:
                 if self.content_server is not None:
-                    # If the content server has any sockets being closed then
+                    # If the Content server has any sockets being closed then
                     # this can take quite a long time (minutes). Tell the user that it is
                     # happening.
                     self.show_shutdown_message(
-                        _('Shutting down the content server. This could take a while ...'))
+                        _('Shutting down the Content server. This could take a while...'))
                     s = self.content_server
                     self.content_server = None
                     s.exit()
